// 
// LeucineZipperPrediction
//
// Erich Bornberg-Bauer and Eric Rivals
//
// release 1.0, 16.6.1999 by Hannes Luz
//
// report problems to <h.luz@dkfz-heidelberg.de>
//

#include <stdio.h>
#include <stdlib.h>

// GNU C++-includes

#include <iostream.h>
#include <fstream.h>
#include <String.h>
#include <new.h>
#include <errno.h>
#include <strstream.h>


const String SPACE = " ";
const int CharPerRow = 60;          // Anzahl der Zeichen pro Zeile im Output (falls verbose-mode)


/*************************************************************************************************************
*****                                                                                                    *****
*****  modify the condition of prediction using Coil: the nb of CCF must not be greater than MAX_NB_CCF  *****
*****        (taken from GenerateSUM.awk)                                                                *****
*****                                                                                                    *****
*************************************************************************************************************/

const int MAX_NB_CCF = 2;		// = 2 -> (hopefully) to get the same reults as IsLZ
const String  MAX_CC_FRAMES = "two";    // fuer output, das muss natuerlich auch an MAX_NB_CCF angepasst werden


#include "LZpred.h"



main(int argc, char *argv[])
{
if (argc == 1)	
	{
	usage(argv[0]);
	exit(1);
	}

struct CC				// eine Struktur, in der die Coiled Coils registriert werden
	{
	int start;
	int length;
	double nb_ccf;
	int in_LZ_id[5];
	int id_counter;
	};

struct LR				// eine Struktur, in der die Leucine Repeats regisrtiert werden
	{
	int start;
	int length;
	int nr; 		        // number of Leucine repeats
	char mut_pos;			// mutation_position

	int in_LZ_id[5];		// = -1		bedeutet, dass das LR in keinem LZ auftaucht
					// = LZ_id	sonst 
	int id_counter;
	};

struct OCL				// eine Struktur, in der die LeucineZipper registriert werden
	{
	int LZ_start;
	int LZ_length;
	char type_of_LR[10];
	int CC_start;
	int CC_length;
	double nb_ccf;
	int LR_start;
	int LR_length;
	};

String FastaDescLine = "";
String Sequence = "";
String CoilResults = "";
String CC_positions[1024];
String CC_pos_buf[1024];
String RegLeucines = "";
String Reason = "";
String InfoLines = "";		// mit InfoLines soll spater das .dat - file aus IsLZ geschrieben werden
				// das ist zwar umstandlich: zuerst alles in einen String zu schreiben,
				// um anschliessend den ganzen String in ein File zu schreiben, aber mir 
				// gefallts besser, da flexibler...

bool there_is_no_LZ = false;
bool already_got_a_reason = false;
bool more_than_3_CC = false;

char coil_extension[256];
char ch;
int nr_of_CC = 0;
int start;
int nr_of_LR;
unsigned int length_of_seq;	
int counter;

// summary-line im .dat-file fehlt noch. Fuer die genaue Beschreibung der Eintage, siehe ~rivals/Awk/Leuzip/GenerateSum.awk ...
bool summary[7];		//  summary[0] - feature line with LR3, LR4, LR5 or Lr_x exists
				//  summary[1] -   "		   LR3   "
				//  summary[2] -   "               LR4   "
				//  summary[3] -   "               LR5   "	
				//  summary[4] -   "               Lr_x   "	
				//  summary[5] - if there is any feature-line with LRx where x > 5
				//  summary[6] - PRED_OCL 

				//
				//	ATTENTION: 
				//
				//	so wie ich es verstanden habe, sind im  OCL-Eintrag im .dat-file alle 
				//      overlaps von LR mit CC aufgefuehrt.
				//
				//	In PRED_OCL steht aber die Anzahl der LeucineZipper!!! (oder, Eric?)
				//


for (int i=0; i<7; i++)		summary[i] = false;

// commandline-flags...

bool VerboseFlag = false;
bool OutFlag = false;
bool InfoFlag = false;
char InFileName[256] = "";

char ** parg= & argv[1];
argc=0;  // to prevent warning

while (*parg)
	{
	if (readArg("-help",parg))		{help(argv[0]); exit(0);}
	if (readArg("-v",parg))			{VerboseFlag=true;continue;}
	if (readArg("-info",parg))		{InfoFlag=true;continue;}
	if (readArg("-o",parg))			{OutFlag=true;continue;}
	if (readArg("-i", &InFileName, parg))	;
	else					{usage(argv[0]); exit(1);}
	}
	
ifstream FastaFile (InFileName);
if (!FastaFile)
	{
	usage(argv[0]);
	error("cannot open the sequence-file", InFileName);
	exit(1);
	}


while ((FastaFile.get(ch)) && (ch != '\n'))	FastaDescLine += ch;
while (FastaFile.get(ch))			
	{
	if (ch <= 'Z' && ch >= 'A')	Sequence += ch;
	else if (ch <= 'z' && ch >= 'a')	
		{
		int delta;
		delta = (int) ('Z' - 'z') + (int) ch;
		ch = (char) delta;
		Sequence += ch;
	       	}
	}

length_of_seq = (unsigned) Sequence.length();



char coil_sequence[256];	
sprintf(coil_sequence,"Coil %s",InFileName);
cout << "Trying to run Coil...\n\n";
system(coil_sequence);
cout << "\n";

sprintf(coil_extension,"%s.cr",InFileName);
ifstream CoilFile (coil_extension);

if (!CoilFile)
	{
	usage(argv[0]);
	error("Seems that running Coil didn't succeed,\ncannot open the file with the results from Coil:", coil_extension);
	exit(1);
      	}


String SeqName[1];
split (InFileName, SeqName, 1, '.'); 
if (SeqName[0].length() > 15)	SeqName[0] = SeqName[0].at(0,15);
Regex SEQ(SeqName[0]);

while (CoilFile.get(ch))	
	{
	if (ch == '\n')
		{
		if (CoilResults.index(SEQ) == -1)
			CoilResults = "";
		else 
			{
			CoilResults += ch;
			break;
			}
		}
	else
		CoilResults += ch;
	}


// ... Eintrag ins .dat-file ... s.o.


CoilResults = CoilResults.after(SeqName[0]);

InfoLines += InFileName;
InfoLines += CoilResults;

CoilResults = CoilResults.after("CCF");
CoilResults.del("     ");


// Gibt es zwischen 1 und 3 coiled coils ??

split(CoilResults, CC_pos_buf, 1024, SPACE);

counter = 0;

for (int i=0; i<1024; i++)
	if (CC_pos_buf[i] != "")
		CC_positions[counter++] = CC_pos_buf[i];

nr_of_CC = atoi(CC_positions[0]);

if (nr_of_CC == 0)	
	{
	already_got_a_reason = true;
	there_is_no_LZ = true;
	Reason = "(no coiled coils detected by Coil)";
	}

if (nr_of_CC > MAX_NB_CCF)
	{
	already_got_a_reason = true;
	there_is_no_LZ = true;
	Reason = "(there were more than ";
	Reason += MAX_CC_FRAMES;
	Reason += " coiled coils detected)";
	more_than_3_CC = true;
//	nr_of_CC = 3;
	}

CC *CoiledCoils = new CC[(nr_of_CC+1)];
for (int i=0; i<=nr_of_CC; i++)		
	{
	CoiledCoils[i].start = -1;
	for (int k=0; k<5; k++)		CoiledCoils[i].in_LZ_id[k] = -1;
	CoiledCoils[i].id_counter = 0;
	}

// if (!there_is_no_LZ)
//	{
	for (int cc = 0; cc < nr_of_CC; cc++)
		{
		CoiledCoils[cc].start = atoi(CC_positions[cc * 3 + 1]);
		int end_of_CC = atoi(CC_positions[cc * 3 + 2]);
		CoiledCoils[cc].length = end_of_CC - CoiledCoils[cc].start + 1;
		CoiledCoils[cc].nb_ccf = atof(CC_positions[cc * 3 + 3]);
		}
//	}




// Die Schleife soll jetzt wie im IsLZ2 - Skript ueber 3 bis 11 
// Leucine Repeats laufen, wobei in der Sequenz zuerst das
// erste vorkommende Leucine-Muster, dann das zweite usw. gesucht
// und registriert werden soll (deshalb SequenceSuffix), was dann auf
// zwei Schleifen rauslauft.

int offset = 0;				// dieser Wert gibt an, ab wo in der Sequenz der regulare
					// Ausdruck gesucht werden soll.

int current_LR_id = 0;
int LR_id;
int start_memory;
int length_memory;

int max_LR;

// in der Sequenz der Lange sequence_length kann es nicht mehr als  
// ( 7 * ((sequence_length / 22) + 1)) LR geben, deshalb:

max_LR = 7 * ( (int)(length_of_seq / 22) + 1);

LR *LR_memory = new LR[max_LR];    // hier sollen die gefundenen Leucine Repeats registriert werden,
			          // der Index ist die ID des Leucine Repeat. 
for (int i=0; i<max_LR; i++)	
	{
	LR_memory[i].start = -1;	// Initialisierung: start = -1		--> kein LR registriert
	for (int k=0; k<5; k++)	LR_memory[i].in_LZ_id[k] = -1;	//     "	  : in_LZ_id = -1	--> noch in keinem LZ registriert
	LR_memory[i].id_counter = 0;
	}

// Hi Eric, ich hab's ausprobiert und gesucht in allen Schreibweisen
// aber nix hat geklappt und ich hab nix gefunden...
// also lass ich alles beim alten

// RegLeucines = "(L......)[2,11]L";
// Regex LRx(RegLeucines);
// start = Sequence.index(LRx);
// cout << start << "\n";
// exit(0);

bool nothing_left_to_search = false;
while (!nothing_left_to_search)
	{
	start_memory = -1;
	
	// ok, jetzt gehts auf die Suche nach den LeucineRepeats
	// RegLeucines ist jetzt "L......L"
	// in der jetzt folgenden Schleife (for (nr_of_LR = 4) wird
	// immer nur nach dem ersten Auftreten eines LR in SequenceSuffix gesucht wird,

	int start_of_first_LR;
	bool LR_search_senseless = false;	
	for (nr_of_LR = 4; (nr_of_LR <= 11) && (!LR_search_senseless); nr_of_LR++)
		{
		if (nr_of_LR == 4)	RegLeucines = "L......L......L";
		RegLeucines += "......L";
		Regex LRx(RegLeucines);
		start = Sequence.index(LRx, offset);
		if (nr_of_LR == 4)	start_of_first_LR = start;
			
      		if (start == -1)	LR_search_senseless =  true;
		else
			{
			bool reg = false;
      			for (LR_id=0; LR_memory[LR_id].start != -1; LR_id++)
				{
				if (LR_memory[LR_id].start == (start+1))    //  wurde das LR schon vorher registriert?
					{
       					LR_memory[LR_id].length = (7* (nr_of_LR-1) + 1);
					LR_memory[LR_id].nr = nr_of_LR;
					length_memory = 7 * (nr_of_LR-1) + 1;
					reg = true;
					}
				}
			if (!reg && start == start_of_first_LR)
				{
	                       	LR_memory[current_LR_id].start = start + 1;
				LR_memory[current_LR_id].length = (7 * (nr_of_LR-1) + 1);
				LR_memory[LR_id].nr = nr_of_LR;
				start_memory = start;
				length_memory = 7 * (nr_of_LR-1) + 1;
				current_LR_id++;
       				}
       			}
		}

	// falls jetzt uberhaupt noch ein LR gefunden wurde, soll bei
	// der nachsten Suche in der Sequenz nach dem in dieser Runde
	// gefundenen LR gesucht werden.
	// Dazu wurde das letzte registrierte Leucine Repeat und dessen Laenge
	// in start_memory und length_memory gespeichert.

	if (start_memory == -1)		nothing_left_to_search = true;	// es wurde keins registriert
	else				offset = (start_memory + length_memory);
       	}


// ... Eintrag ins .dat-file ... s.o.
// 
// wie gesagt: umstandlich, aber flexibel...
// und ich will das Format aus IsLZ, also

char nr_of_nr_of_LR[12];
for (int i=0; i<12; i++)	nr_of_nr_of_LR[i] = 0;	 

for (int j=4; j<=11; j++) 
	for (int i=0; LR_memory[i].start != -1; i++)	
		if (j == LR_memory[i].nr)		nr_of_nr_of_LR[j]++;

int nr_of_all_LR = 0;
for (int j=4; j<=11; j++)	nr_of_all_LR += nr_of_nr_of_LR[j];

for (int j=4; j<=11; j++)
	{
	if (nr_of_nr_of_LR[j] != 0)
		{
       		char string[256] = "";
		sprintf(string,"%s            LR%d",InFileName,j);
		InfoLines += string;
		sprintf(string,"     %d",nr_of_nr_of_LR[j]);
		InfoLines += string;
		
		for (int i=0; LR_memory[i].start != -1; i++)
			if (j == LR_memory[i].nr)
				{
				sprintf(string, "   %d    %d", LR_memory[i].start, (LR_memory[i].start + LR_memory[i].length - 1));
				InfoLines += string;
				}
		InfoLines += "\n";

		// hier kommen die Eintrage in Erics summary-line

		if (j <= 5)	
			{
			summary[0] = true;
			summary[j-2] = true;
			}
		else	summary[5] = true;
		}
       	}

// jetzt noch feststellen, ob es eine Stelle Stelle mit 5 Lrepeats   
// gibt, wobei ein L durch [MIV] ersetzt sein darf 

int max_Lrx;

// in der Sequenz der Lange sequence_length kann es nicht mehr als  
// ( 7 * ((sequence_length / 29) + 1)) Lr_x geben, deshalb:

max_Lrx = 7 * ( (int)(length_of_seq / 29) + 1);

LR *LRmut_memory = new LR[max_Lrx];	// hier sollen die gefundenen Leucine Repeats mit Mutationen registriert werden,
					// der Index ist die ID des Leucine Repeat. 
			          
int id = 0;
for (int i=0; i<max_Lrx; i++)	
	{
	LRmut_memory[i].start = -1;		// Initialisierung: start = -1		--> kein LRmut registriert
	for (int k=0; k<5; k++)	LRmut_memory[i].in_LZ_id[k] = -1;		//        "       : in_LZ_id = -1   --> noch in keinem LZ registriert
	LRmut_memory[i].id_counter = 0;
	}

// erst mal die regex zusammen basteln

for (int mutation_position = 1; mutation_position <= 5; mutation_position++)
	{
	RegLeucines = "";
	for (counter=1; (mutation_position-counter) > 0; counter++)	RegLeucines += "L......";
	if (! (mutation_position==5))					
		{
		RegLeucines += "[MIV]......";
		for (counter=mutation_position+1; counter <= 5; counter++)
			{
			if (! (counter == 5))	RegLeucines += "L......";
			else			RegLeucines += "L";
			}
		}
	else					RegLeucines += "[MIV]";

	//       	cout << RegLeucines << "\n"; 
	//}
	// hier ist das RegLeucine der String mit funf Leucine-Repeats und einer Mutation des 'mutation_position'-ten L.

	Regex LRmut(RegLeucines);
	
	offset = 0;
	nothing_left_to_search = false;	

	while (! nothing_left_to_search)
		{
		start = Sequence.index(LRmut,offset);
		if (start == -1)		nothing_left_to_search = true;
		else	
			{
			LRmut_memory[id].start = start + 1;
			LRmut_memory[id].length = 4*7 + 1; 
			LRmut_memory[id].mut_pos = (char) mutation_position; 
			offset += (LRmut_memory[id].start + LRmut_memory[id].length);
			id++;
       			}
		}
	}




// ... Eintrag ins .dat-file ... s.o.

int nr_of_Lr_x[6];
for (int i=0; i<6; i++)		nr_of_Lr_x[i] = 0;	 

for (int j=1; j<=5; j++) 
	for (int i=0; LRmut_memory[i].start != -1; i++)	
		if (j == (int) LRmut_memory[i].mut_pos)		nr_of_Lr_x[j]++;

for (int j=1; j<=5; j++)	nr_of_all_LR += nr_of_Lr_x[j];

for (int j=1; j<=5; j++)
	{
	if (nr_of_Lr_x[j] != 0)
		{
		char string[256] = "";
		sprintf(string,"%s            Lr4_%d",InFileName,j);
		InfoLines += string;
		sprintf(string,"   %d", nr_of_Lr_x[j]);
		InfoLines += string;
		
		for (int i=0; LRmut_memory[i].start != -1; i++)
			if (j == (int) LRmut_memory[i].mut_pos)
				{
				sprintf(string, "   %d    %d", LRmut_memory[i].start, (LRmut_memory[i].start + LRmut_memory[i].length - 1));
				InfoLines += string;
				}

		// fur die summary-line im InfoFile .dat:
		summary[4] = true;

		InfoLines += "\n";
		}
       	}


/**************************************************************************
********  ok, jetzt Uberlapp zwischen Lrepeats und Coiled Coil   **********
********  detektieren und registrieren:                          **********
********							 **********
********  a) falls mehr als 3 CC oder falls das mit einem Lr     **********
********     uberlappnede CC mehr als 90 residues hat            **********
********     --> nix registrieren                                **********
********  b) falls CC mindestens 21 residues Uberlapp            **********
********     mit LRX bzw Lr4_x hat --> registrieren              **********
****************************************************************************/


int LZ_id = 0;
int nr_of_LZ = 0;
int max_LZ;	// da es nicht mehr als (nr_of_all_LR * nr_of_CC) LeucineZipper geben kann

max_LZ = nr_of_all_LR * nr_of_CC;

OCL *LZ = new OCL[max_LZ];				// hier sollen die LeucineZipper gespeichert werden
for (int i=0; i<max_LZ; i++)    LZ[i].LZ_start = -1;

if (nr_of_CC == 0)	
	{
	char string[256];
	sprintf(string,"%s            OCL   0\n",InFileName);
	InfoLines += string;
	}



else			// Uberlapp von CC mit den LRs testen
	{
	bool all_CCs_too_long = true;

	for (int cc = 0; cc < nr_of_CC; cc++)
		{
		int overlap;
       		int start_of_CC = atoi(CC_positions[cc * 3 + 1]);
		int end_of_CC = atoi(CC_positions[cc * 3 + 2]);
		int length_of_CC = end_of_CC - start_of_CC + 1;
		double nb_ccf = atof(CC_positions[cc * 3 + 3]);

		if ((end_of_CC - start_of_CC) <= 90)
			{
			all_CCs_too_long = false;

			for (int i=0; LR_memory[i].start != -1; i++)
				{
				if ((overlap = overlap_CC_LR(start_of_CC, length_of_CC, LR_memory[i].start, LR_memory[i].length)) >= 21)
					{
					CoiledCoils[cc].in_LZ_id[CoiledCoils[cc].id_counter++] = LZ_id;
					LR_memory[i].in_LZ_id[LR_memory[i].id_counter++] = LZ_id;

					if (start_of_CC >= LR_memory[i].start)	LZ[LZ_id].LZ_start = start_of_CC;
					else					LZ[LZ_id].LZ_start = LR_memory[i].start;
					LZ[LZ_id].LZ_length = overlap;
					sprintf(LZ[LZ_id].type_of_LR, "LR%d",LR_memory[i].nr);
					LZ[LZ_id].CC_start = start_of_CC;
					LZ[LZ_id].CC_length = length_of_CC;
					LZ[LZ_id].LR_start = LR_memory[i].start;
					LZ[LZ_id].LR_length = LR_memory[i].length;
					LZ[LZ_id].nb_ccf = nb_ccf;
					nr_of_LZ++;
					LZ_id++;
					}
				}
			for (int i=0; LRmut_memory[i].start != -1; i++)
				{
				if ((overlap = overlap_CC_LR(start_of_CC, length_of_CC, LRmut_memory[i].start, LRmut_memory[i].length)) >= 21)
					{
					CoiledCoils[cc].in_LZ_id[CoiledCoils[cc].id_counter++] = LZ_id;
					LRmut_memory[i].in_LZ_id[LRmut_memory[i].id_counter++] = LZ_id;

					if (start_of_CC >= LRmut_memory[i].start)	LZ[LZ_id].LZ_start = start_of_CC;
					else						LZ[LZ_id].LZ_start = LRmut_memory[i].start;
					LZ[LZ_id].LZ_length = overlap;
					sprintf(LZ[LZ_id].type_of_LR, "Lr4_%d",LRmut_memory[i].mut_pos);
					LZ[LZ_id].CC_start = start_of_CC;
					LZ[LZ_id].CC_length = length_of_CC;
					LZ[LZ_id].LR_start = LRmut_memory[i].start;
					LZ[LZ_id].LR_length = LRmut_memory[i].length;
					LZ[LZ_id].nb_ccf = nb_ccf;
					nr_of_LZ++;
					LZ_id++;
					}
				}
			}
		}

	if ((all_CCs_too_long) && (!already_got_a_reason))
		{
		there_is_no_LZ = true;
		if (nr_of_CC > 1)	Reason = "(the Coiled Coils contain more than 90 residues)";
		if (nr_of_CC == 1)	Reason = "(the Coiled Coil contains more than 90 residues)";
		already_got_a_reason = true;
		} 



	//
	// so: jetzt sind alle OCLs im Kasten (soferns welche gab, haha).
	//
	// 
	//
	// jetzt gehts wieder an das lastige .dat-File
	//

	char string[256];
	sprintf(string,"%s            OCL     %d",InFileName,nr_of_LZ);
	InfoLines += string;
	
        for (int i=0; i < nr_of_LZ; i++)
		{
		summary[6] = true;

		sprintf(string, "    %d    %d %.2f  %d %s", LZ[i].LZ_start, (LZ[i].LZ_start + LZ[i].LZ_length - 1), LZ[i].nb_ccf, LZ[i].LZ_length, LZ[i].type_of_LR);
		InfoLines += string;
		}
	InfoLines += "\n";
       	}


if ((nr_of_LZ == 0) && (!already_got_a_reason))
	{
	there_is_no_LZ = true;

	// gibts uberhaupt ein LeucineRepeat ?
		
	if ((LR_memory[0].start == -1) && (LRmut_memory[0].start == -1))
		{
		Reason = "there were no Leucine Repeats detected.";
		already_got_a_reason = true;
		}
	else
		{
		Reason = "the condition of a minimum overlap of 21 residues\n betweeen the coiled coil and the Leucine Repeat\nisn't fulfilled.";
		already_got_a_reason = true;
		}
	}




// summary-line im .dat-file fehlt noch. Fuer die genaue Beschreibung der Eintage, siehe ~rivals/Awk/Leuzip/GenerateSum.awk ...

char string[256];
sprintf(string,"%s            SUM   --- ---    -1 -1", InFileName);
InfoLines += string;
for (int i=0; i<6; i++)
	{
	sprintf(string, " %d", summary[i]);
	InfoLines += string;
	}


sprintf(string, " 0 0 0 0  0 0.00 ----- 0  %d", nr_of_CC);
// sprintf(string, " 0 0 0 0  0 0.00 ----- 0  %d", summary[6]);
InfoLines += string;
if (LZ[0].LZ_start == -1)	InfoLines += " 0.00 -----  0\n";	 // kein OCL --> kein LZ
else if (! more_than_3_CC)
	{
	sprintf(string, " %.2f %s    %d\n", LZ[0].nb_ccf, LZ[0].type_of_LR, nr_of_LZ);
	InfoLines += string;
	}

else
	{
	sprintf(string, " %.2f %s    0\n", LZ[0].nb_ccf, LZ[0].type_of_LR);
	InfoLines += string;
	}



//
// so: der Output muss auch noch zusammengfrickelt werden.
//
// die Info, die in jedem Fall kommen soll: gabs ein LZ oder
// nicht, und wenn nicht, warum nicht.
//

if (there_is_no_LZ)
	{
	cout << "The protein sequence in " << InFileName << " is predicted not to contain a Leucine Zipper domain.\n"; 
	if (already_got_a_reason)	cout << Reason << "\n";
	//	else				cout << ".\n";
	}

else
	{
	cout << "The protein sequence in " << InFileName << " is predicted to contain a Leucine Zipper domain\n";
	cout << "between positions " << LZ[0].LZ_start << " and " << (LZ[0].LZ_start + LZ[0].LZ_length - 1) << ".\n";
	}

cout << "\n\n";

if (VerboseFlag)
	{
	int start_of_LR;
	int nr_of_L;
	int start_of_CC;
	int length_of_CC;
	int LZ_id = 0;

	cout << "verbose mode is on:\n\n";
	cout << "description line:\n" << FastaDescLine << "\n\n";


// 
// gleich kommt Flickwerk, nr_of_LZ ist nur dann die Anzahl der LZ, falls nr_of_CC <= MAX_NB_CCF   !
//

	cout << "1) number of potential LEUCINE ZIPPERS: "; 
	cout << nr_of_LZ << "\n";

	if (there_is_no_LZ && nr_of_LZ > 0)	
		{
		cout << "\nA prediction of Leucine Zippers was rejected.\n";
		cout << Reason << ".\n";
		cout << "However the overlaps  between CoiledCoil frames and\n";
		cout << "Leucine repeats were detected as follows:\n\n\n"; 
		}



	while (LZ_id < nr_of_LZ)
		{

		if (! there_is_no_LZ)		// flick, flick
		{
		cout << "\n";
		bool finished = false;
		for (int cc = 0; cc < MAX_NB_CCF && CoiledCoils[cc].start != -1 && !finished; cc++)
			{
			for (int k=0; k<5 && !finished; k++)
				{
				if (CoiledCoils[cc].in_LZ_id[k] == LZ_id)
					{
					start_of_CC = CoiledCoils[cc].start;
					length_of_CC = CoiledCoils[cc].length;
					finished = true;
					}
				}
			}

		bool LR_is_in_LZ = false;
		finished = false;

 		for (LR_id = 0; LR_memory[LR_id].start != -1 && !finished; LR_id++)
			{
			for (int k=0; k<5 && !finished; k++)
				{
				if (LR_memory[LR_id].in_LZ_id[k] == LZ_id)
					{
					start_of_LR = LR_memory[LR_id].start;
					nr_of_L = LR_memory[LR_id].nr;
					LR_is_in_LZ = true;
					finished = true;
					}						
				}
			}

		finished = false;
		
		if (!LR_is_in_LZ)
			{
			char mutation_position;

			for (int Lr_id = 0; LRmut_memory[Lr_id].start != -1; Lr_id++)
				{
				for (int k=0; k<5 && !finished; k++)
					{
					if (LRmut_memory[Lr_id].in_LZ_id[k] == LZ_id)
						{
						start_of_LR = LRmut_memory[Lr_id].start;
						mutation_position = LRmut_memory[Lr_id].mut_pos;
						finished = true;
						}
					}
				}
			annotation(Sequence, start_of_CC, length_of_CC, start_of_LR, mutation_position, LZ[LZ_id].LZ_start, LZ[LZ_id].LZ_length);
			}
		else
			annotation(Sequence, start_of_CC, length_of_CC, start_of_LR, nr_of_L, LZ[LZ_id].LZ_start, LZ[LZ_id].LZ_length);
		
		LZ_id++;
		}		
		
		else	
		{

//	      
//	ok, hier wird jetzt noch der Fall beruecksichtigt, dass das (die) OCL kein(e) LZ ist(sind),
//
//	da es zu viele CC-frames (mehr als MAX_NB_CCF (s.o.) gab.
//


		bool finished = false;
		for (int cc = 0; cc < nr_of_CC && CoiledCoils[cc].start != -1 && !finished; cc++)
			{
			for (int k=0; k<5 && !finished; k++)
				{
				if (CoiledCoils[cc].in_LZ_id[k] == LZ_id)
					{
					start_of_CC = CoiledCoils[cc].start;
					length_of_CC = CoiledCoils[cc].length;
					finished = true;
					}
				}
			}

		bool LR_is_in_LZ = false;
		finished = false;


 		for (LR_id = 0; LR_memory[LR_id].start != -1 && !finished; LR_id++)
			{
			for (int k=0; k<5 && !finished; k++)
				{
				if (LR_memory[LR_id].in_LZ_id[k] == LZ_id)
					{
					start_of_LR = LR_memory[LR_id].start;
					nr_of_L = LR_memory[LR_id].nr;
					LR_is_in_LZ = true;
					finished = true;
					}						
				}
			}

		finished = false;

       		if (!LR_is_in_LZ)
			{
			char mutation_position;

			for (int Lr_id = 0; LRmut_memory[Lr_id].start != -1; Lr_id++)
				{
				for (int k=0; k<5 && !finished; k++)
					{
					if (LRmut_memory[Lr_id].in_LZ_id[k] == LZ_id)
						{
						start_of_LR = LRmut_memory[Lr_id].start;
						mutation_position = LRmut_memory[Lr_id].mut_pos;
						finished = true;
						}
					}
				}
			OCL_annotation(Sequence, start_of_CC, length_of_CC, start_of_LR, mutation_position, LZ[LZ_id].LZ_start, LZ[LZ_id].LZ_length);
			}
		else
			OCL_annotation(Sequence, start_of_CC, length_of_CC, start_of_LR, nr_of_L, LZ[LZ_id].LZ_start, LZ[LZ_id].LZ_length);
		

		LZ_id++;
		}

// Ende der OCL-Annotation 


		}





	cout << "\n\n\n";
	cout << "2) COILED COILS which do NOT correspond to a Leucine Zipper\n\n";

	bool no_CC_verbosed = true;

	if (!more_than_3_CC)
	  {
        for (int cc_id=0; (cc_id < 3 && CoiledCoils[cc_id].start != -1); cc_id++)
		{
		if (CoiledCoils[cc_id].in_LZ_id[0] == -1)
			{
			CC_annotation(Sequence, CoiledCoils[cc_id].start, CoiledCoils[cc_id].length);
			no_CC_verbosed = false;
			}
		}
	  }

	if (no_CC_verbosed)	
		{
		if (more_than_3_CC)
			{
			cout << "Coil detected more than ";
			cout << MAX_CC_FRAMES;
			cout << " Coiled Coils.\n";
			cout << "Looking for Leucine Zippers seems senseless.\n";
			cout << "And that's why here are no Coiled Coils annotated...\n";
			if (InfoFlag)
				cout << "Look at the InfoLines for more details\n";
			else if (OutFlag)
				cout << "Have a look at the outfile " << InFileName << ".dat for more details\n";
			else 
				cout << "Set the info-flag (-info) or the outflag (-o) for more details.\n";
			}
		else	cout << "none\n";
		}

	cout << "\n\n\n";
	cout << "3) LEUCINE REPEATS which do NOT correspond to a Leucine Zipper\n\n";

	bool no_LR_verbosed = true;
	
	for (LR_id = 0; (LR_id < max_LR && LR_memory[LR_id].start != -1); LR_id++)
		{
		if (LR_memory[LR_id].in_LZ_id[0] == -1)
			{
			annotation(Sequence, LR_memory[LR_id].start, LR_memory[LR_id].nr);
			no_LR_verbosed = false;
			}
		}
	for (int Lr_id = 0; (Lr_id < max_Lrx && LRmut_memory[Lr_id].start != -1); Lr_id++)
		{
		if (LRmut_memory[Lr_id].in_LZ_id[0] == -1)
			{
			annotation(Sequence, LRmut_memory[Lr_id].start, LRmut_memory[Lr_id].mut_pos);
			no_LR_verbosed = false;
			}
		}
	if (no_LR_verbosed)	cout << "none\n";

	cout << "\n\n";	
	}



if (InfoFlag)
	{
	cout << "Info-mode is on:\n\n";
	cout << InfoLines;
	cout << "\n\n";
	}


if (OutFlag)
	{
	char outfile[128];
	sprintf(outfile, "%s.dat", InFileName);
	ofstream out(outfile);
	if (!out)
		{
		error("cannot open the outfile", outfile);
		exit(1);
		}

      	cout << "writing infolines to file " << outfile << "\n";

	out << InfoLines;
	}

exit(0);
}

