void usage(char string[])
{
cerr << "Usage: " << string << " [-help] <-i inputfile with fasta-sequence> [-v] [-info] [-o]\n";
}

void help(char string[])
{
usage(string);
cout << "\n<-i inputfile with fasta-sequence> : i.e.: LZ -i sequence.fasta\n\n";
cout << "optional flags:\n";
cout << "[-help]\tprints this help on stdout only.\n";
cout << "[-v]\tprints annotations\n\tof all detected Coiled Coils, Leucine Repeats and Leucine Zippers\n\ton stdout\n";
cout << "[-info] prints some InfoLines (Coiled Coil, Leucine Repeats,\n\tLeucine Zipper, Summary Line) on stdout\n";
cout << "[-o]\twrites a file named inputfile.dat containing the InfoLines\n";
cout << "\n\n\n";
cout << "The follwing lines give an example for an annotation:\n";
cout << "1) number of potential LEUCINE ZIPPERS: 1                         Annotations:\n";
cout << "                                                                 |\n";
cout << "1---------11--------21--------31--------41--------51--------     |\n";
cout << "MVVVAAAPNPADGTPKVLLLSGQPASAAGAPAARLPLMVPAQRGASPEAASGGLPQARKR     |\n";
cout << "                                                                 |\n";
cout << "                                                                 |\n";
cout << "                                                                 |\n";
cout << "                                                                 |\n";
cout << "61--------71--------81--------91--------101-------111-------     |position \n";
cout << "QRLTHLSPEEKALRRKLKNRVAAQTARDRKKARMSELEQQVVDLEEENQKLLLENQLLRE     |sequence\n";
cout << "                CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC     |coiled coil \n";
cout << "                                    L------L------L------L--     |L-Repeats\n";
cout << "                                    LZLZLZLZLZLZLZLZLZLZLZLZ     |LeucineZipper\n";
cout << "                                                                 |\n";
cout << "121-------131-------141-------151-------161-------171-------     |\n";
cout << "KTHGLVVENQELRQRLGMDALVAEEEAEAKGNEVRPVAGSAESAALRLRAPLQQVQAQLS     |\n";
cout << "CCCCCCCCCCCC                                                     |\n";
cout << "----L------L                                                     |\n";
cout << "LZLZLZLZLZLZ                                                     |\n";
cout << "                                                                 |\n";
cout << "181-------191-------201-------211-------221-------231-------     |\n";
cout << "PLQNISPWILAVLTLQIQSLISCWAFWTTWTQSCSSNALPQSLPAWRSSQRSTQKDPVPY     |\n";
cout << "                                                                 |\n";
cout << "                                                                 |\n";
cout << "                                                                 |\n";
cout << "                                                                 |\n";
cout << "241-------251-------                                             |\n";
cout << "QPPFLCQWGRHQPSWKPLMN                                             |\n";
cout << "\n";
cout << "report problems to <h.luz@dkfz-heidelberg.de>\n";
}

void error (char *s, char *s2="")
{
cerr << s << ' ' << s2 << "\n";
perror("");
}

// gibt die Anzahl der Dezimalstellen zuruck

int stellen(int position)
{
char pos[32];
int k;
		
sprintf(pos,"%d",position);
for (k=0; pos[k] != '\0'; k++)	;
return(k);
}


// berechnet den Uberlapp zwischen einem Coiled Coil und einem LeucineRepeat
// und gibt die Grosse des Uberlapps zuruck

int overlap_CC_LR(int cc_start, int cc_length, int lr_start, int lr_length)
{
int overlap;

if (lr_start < cc_start)
	{
	  if ((lr_start + lr_length - 1) < cc_start)			overlap = 0; 
	  else								overlap = (lr_start + lr_length) - cc_start; 
	}
else
	{
	  if ((cc_start + cc_length - 1) < lr_start)			overlap = 0; 
	  else if ((cc_start + cc_length) >= (lr_start + lr_length))	overlap = lr_length;
	else								overlap = (cc_start + cc_length) - lr_start; 
	}

return(overlap);
}

//
// jetzt ein paar Funktionen fur den Output, falls der verbose-mode [-v] an ist.
//

// annotation for a LZ with LRs
void annotation(String Sequence, int CC_start, int CC_length, int LR_start, int nr_of_L, int LZ_start, int LZ_length)
{
int length;
int counter = 0;
int position = 0;

length = Sequence.length();

String Leucine_Pattern = "";

for (int i=1; i<LR_start; i++)					Leucine_Pattern += " ";
for (int i=1; i<nr_of_L; i++)					Leucine_Pattern += "L------";
Leucine_Pattern += "L";
for (int i=0; i<=(length-(LR_start+(7*(nr_of_L-1)+1))); i++)	Leucine_Pattern += " ";


String LZ_Pattern = "";
for (int i=1; i<LZ_start; i++)				LZ_Pattern += " ";
for (int i=1; i<LZ_length;)			
	{
	LZ_Pattern += "LZ";
	i += 2;
	if (i == LZ_length)		LZ_Pattern += "L";
	}
for (int i=0; i<(length-(LZ_start+LZ_length-1)); i++)	LZ_Pattern += " ";

while (counter < length)
	{
       	int rest;
	int row_length = CharPerRow;

	rest = length - counter;
	if (rest < CharPerRow)		row_length = rest;

// hier kommen die Positionen

	int position_merker = position;

	while ((position-position_merker) < row_length)
		{
		int i;
		position++;
		printf("%d", position);
		position += stellen(position);
		for (i = stellen(position); i<9 && ((position-position_merker) < row_length); i++)	
			{
			putchar('-');
			position++;
			}
		if ((position-position_merker) <= row_length || i == 9)		putchar('-');
		}
	putchar('\n');

	position = position_merker;

// und dazu die Sequenz

	cout << Sequence.at(counter, row_length) << "\n";

// so, und jetzt das CoiledCoil

	for (int i=0; i<row_length; i++)
		{
		position++;
		if (position >= CC_start && position <= (CC_start+CC_length-1))	putchar('C');
		else								putchar(' ');
		}
	putchar('\n');

// ... Leucine repeats

	cout << Leucine_Pattern.at(counter, row_length) << "\n";

// ... Leucine Zipper annotation

	cout << LZ_Pattern.at(counter, row_length) << "\n\n";

	counter += CharPerRow;
	}
} 			
		 
// annotation for a LZ with LRs and a mutation

void annotation(String Sequence, int CC_start, int CC_length, int Lr_start, char mut_pos, int LZ_start, int LZ_length)
{
int length;
int counter = 0;
int position = 0;

length = Sequence.length();

String Leucine_Pattern = "";
for (int i=1; i<Lr_start; i++)				Leucine_Pattern += " ";

for (char i=1; i<mut_pos; i++)				Leucine_Pattern += "L------";
Leucine_Pattern += Sequence.at((Lr_start + (7 * ((int)mut_pos-1)))-1, 1);
for (char i = mut_pos; i<5; i++)			Leucine_Pattern += "------L";
for (int i=0; i<=(length-(Lr_start+(7*4+1))); i++)	Leucine_Pattern += " ";


String LZ_Pattern = "";
for (int i=1; i<LZ_start; i++)				LZ_Pattern += " ";
for (int i=1; i<LZ_length;)			
	{
	LZ_Pattern += "LZ";
	i += 2;
	if (i == LZ_length)		LZ_Pattern += "L";
	}
for (int i=0; i<(length-(LZ_start+LZ_length-1)); i++)	LZ_Pattern += " ";


while (counter < length)
	{
       	int rest;
	int row_length = CharPerRow;

	rest = length - counter;
	if (rest < CharPerRow)		row_length = rest;
		
// hier kommen die Positionen

	int position_merker = position;

	while ((position-position_merker) < row_length)
		{
		int i;
		position++;
		printf("%d", position);
		position += stellen(position);
		for (i = stellen(position); i<9 && ((position-position_merker) < row_length); i++)	
			{
			putchar('-');
			position++;
			}
		if ((position-position_merker) <= row_length || i == 9)	putchar('-');
		}
	putchar('\n');

	position = position_merker;

// und dazu die Sequenz

	cout << Sequence.at(counter, row_length) << "\n";

// so, und jetzt das CoiledCoil

	for (int i=0; i<row_length; i++)
		{
		position++;
		if (position >= CC_start && position <= (CC_start+CC_length-1))	putchar('C');
		else								putchar(' ');
		}
	putchar('\n');

// ... Leucine repeats mit einer Mutation

	cout << Leucine_Pattern.at(counter, row_length) << "\n";

// ... Leucine Zipper annotation

	cout << LZ_Pattern.at(counter, row_length) << "\n\n";

	counter += CharPerRow;
	}
}


// annotation for a CC without LZ

void CC_annotation(String Sequence, int CC_start, int CC_length)
{
int length;
int counter = 0;
int position = 0;

length = Sequence.length();

while (counter < length)
	{
       	int rest;
	int row_length = CharPerRow;

	rest = length - counter;
	if (rest < CharPerRow)		row_length = rest;

// hier kommen die Positionen

	int position_merker = position;

	while ((position-position_merker) < row_length)
		{
		int i;
		position++;
		printf("%d", position);
		position += stellen(position);
		for (i = stellen(position); i<9 && ((position-position_merker) < row_length); i++)	
			{
			putchar('-');
			position++;
			}
		if ((position-position_merker) <= row_length || i == 9)		putchar('-');
		}
	putchar('\n');

	position = position_merker;

// und dazu die Sequenz

	cout << Sequence.at(counter, row_length) << "\n";

// so, und jetzt das CoiledCoil

	for (int i=0; i<row_length; i++)
		{
		position++;
		if (position >= CC_start && position <= (CC_start+CC_length-1))	putchar('C');
		else								putchar(' ');
		}
	putchar('\n');
	putchar('\n');

	counter += CharPerRow;
	}
} 	


// annotation for a LR without LZ

void annotation(String Sequence, int LR_start, int nr_of_L)
{
int length;
int counter = 0;
int position = 0;

length = Sequence.length();

String Leucine_Pattern = "";
for (int i=1; i<LR_start; i++)					Leucine_Pattern += " ";
for (int i=1; i<nr_of_L; i++)					Leucine_Pattern += "L------";
Leucine_Pattern += "L";
for (int i=0; i<=(length-(LR_start+(7*(nr_of_L-1)+1))); i++)	Leucine_Pattern += " ";


while (counter < length)
	{
       	int rest;
	int row_length = CharPerRow;

	rest = length - counter;
	if (rest < CharPerRow)		row_length = rest;

// hier kommen die Positionen

	int position_merker = position;

	while ((position-position_merker) < row_length)
		{
		int i;
		position++;
		printf("%d", position);
		position += stellen(position);
		for (i = stellen(position); i<9 && ((position-position_merker) < row_length); i++)	
			{
			putchar('-');
			position++;
			}
		if ((position-position_merker) <= row_length || i == 9)		putchar('-');
		}
	putchar('\n');

// und dazu die Sequenz

	cout << Sequence.at(counter, row_length) << "\n";
    
// ... Leucine repeats

	cout << Leucine_Pattern.at(counter, row_length) << "\n\n";

	counter += CharPerRow;
	}
} 

// annotation for a Lr (mutation) without LZ

void annotation(String Sequence, int Lr_start, char mut_pos)
{
int length;
int counter = 0;
int position = 0;

length = Sequence.length();

String Leucine_Pattern = "";
for (int i=1; i<Lr_start; i++)				Leucine_Pattern += " ";

for (char i=1; i<mut_pos; i++)				Leucine_Pattern += "L------";
Leucine_Pattern += Sequence.at((Lr_start + (7 * ((int)mut_pos-1)))-1, 1);
for (char i = mut_pos; i<5; i++)			Leucine_Pattern += "------L";
for (int i=0; i<=(length-(Lr_start+(7*4+1))); i++)	Leucine_Pattern += " ";


while (counter < length)
	{
       	int rest;
	int row_length = CharPerRow;

	rest = length - counter;
	if (rest < CharPerRow)		row_length = rest;
		
// hier kommen die Positionen

	int position_merker = position;

	while ((position-position_merker) < row_length)
		{
		int i;
		position++;
		printf("%d", position);
		position += stellen(position);
		for (i = stellen(position); i<9 && ((position-position_merker) < row_length); i++)	
			{
			putchar('-');
			position++;
			}
		if ((position-position_merker) <= row_length || i == 9)		putchar('-');
		}
	putchar('\n');

// und dazu die Sequenz

	cout << Sequence.at(counter, row_length) << "\n";

// ... Leucine repeats mit einer Mutation

	cout << Leucine_Pattern.at(counter, row_length) << "\n\n";

	counter += CharPerRow;
	}
}


//
//
// the following code was kindly provided by 
//
// Rainer Heintzmann, Institut fuer Angewandte Physik Heidelberg
//
//

#ifndef parseargs_h		// -*- C++ -*- 
#define parseargs_h

void printCall(int argc, char * argv[])
{
  int i;
  cout << "Call was :\n";
  for (i=0;i< argc;i++)
    {
      // cout << "\n" << i << "  ";
      cout << argv[i] << " ";
    }
  cout << "\n";
}

/// just parse a tag-string and return true if it is there.

int readArg(const char * TagString, char ** &argv)
{
    if (strcmp(* argv,TagString) != 0)
      return 0;
    else
      argv ++;
 //   cout << " Parsed switch tag\n";
    return 1;
}
 

/// parses a tag-string and any following variable-type into the variable Value. returns 1(true) if sucessful
template <class Targ>
int readArg(const char * TagString, Targ * Value, char ** &argv)
{
  char ** valwhere= argv;
  char tstchar;

  if (* valwhere == 0) return 0;

  if (TagString)
    if (strcmp(* argv,TagString) != 0)
      {if (* TagString) return 0;}
    else
      valwhere ++;

//  cout << " Parsed tag\n";
  if (* valwhere == 0) return 0;

  istrstream ist(* valwhere);

  if (! (ist >> (* Value)))
     return 0;

  // cout << " Parsed Value : " << (* Value) << "\n";

  if (ist >> tstchar)
    return 0;

//  cout << " Parsed succesfull\n";
  if (* TagString) argv += 2;
  else argv += 1;
  return 1;         // successfully parsed argument
}

int readArg(const char * TagString, char * Value, char ** &argv)  // special version for Strings reading the whole string
{
  char ** valwhere= argv;

  if (* valwhere == 0) return 0;

  if (TagString)
    if (strcmp(* argv,TagString) != 0)
      {if (* TagString) return 0;}
    else
      valwhere ++;

  // cout << " Parsed tag\n" << flush;
  if (* valwhere == 0) return 0;

  strcpy(Value,* valwhere);

  // printf(" Parsed Value : %s\n",Value);

//  cout << " Parsed succesfull\n";
  if (* TagString) argv += 2;
  else argv += 1;
  return 1;         // successfully parsed argument
}

template <class Targ>
char * getVal(const char * tag, const char * filestr,Targ * Value)
{
  char * parse;
  parse=strstr(filestr,tag);
  if (! parse) 
    {
      cerr << "Fatal error: File does not contain tag " << tag << " \n";
      exit(-1);
    }
  istrstream ist(parse+strlen(tag));
  if (! (ist >> (* Value)))
    {
      cerr << "Fatal error: Value after tag '" << tag << "' has wrong dataformat !\n";
      exit(-1);
    }

  return parse;

}

#endif


//
// end of Rainer Heintzmanns Code
//

// annotation for a OCL with LRs
void OCL_annotation(String Sequence, int CC_start, int CC_length, int LR_start, int nr_of_L, int LZ_start, int LZ_length)
{
int length;
int counter = 0;
int position = 0;

length = Sequence.length();

String Leucine_Pattern = "";

for (int i=1; i<LR_start; i++)					Leucine_Pattern += " ";
for (int i=1; i<nr_of_L; i++)					Leucine_Pattern += "L------";
Leucine_Pattern += "L";
for (int i=0; i<=(length-(LR_start+(7*(nr_of_L-1)+1))); i++)	Leucine_Pattern += " ";

String LZ_Pattern = "";
for (int i=1; i<LZ_start; i++)				LZ_Pattern += " ";
for (int i=1; i<=LZ_length; i++)			
	LZ_Pattern += "O";
for (int i=0; i<(length-(LZ_start+LZ_length-1)); i++)	LZ_Pattern += " ";

while (counter < length)
	{
       	int rest;
	int row_length = CharPerRow;

	rest = length - counter;
	if (rest < CharPerRow)		row_length = rest;

// hier kommen die Positionen

	int position_merker = position;

	while ((position-position_merker) < row_length)
		{
		int i;
		position++;
		printf("%d", position);
		position += stellen(position);
		for (i = stellen(position); i<9 && ((position-position_merker) < row_length); i++)	
			{
			putchar('-');
			position++;
			}
		if ((position-position_merker) <= row_length || i == 9)		putchar('-');
		}
	putchar('\n');

	position = position_merker;

// und dazu die Sequenz

	cout << Sequence.at(counter, row_length) << "\n";

// so, und jetzt das CoiledCoil

	for (int i=0; i<row_length; i++)
		{
		position++;
		if (position >= CC_start && position <= (CC_start+CC_length-1))	putchar('C');
		else								putchar(' ');
		}
	putchar('\n');

// ... Leucine repeats

	cout << Leucine_Pattern.at(counter, row_length) << "\n";

// ... Leucine Zipper annotation

	cout << LZ_Pattern.at(counter, row_length) << "\n\n";

	counter += CharPerRow;
	}
} 			
		 
// annotation for a LZ with LRs and a mutation

void OCL_annotation(String Sequence, int CC_start, int CC_length, int Lr_start, char mut_pos, int LZ_start, int LZ_length)
{
int length;
int counter = 0;
int position = 0;

length = Sequence.length();

String Leucine_Pattern = "";
for (int i=1; i<Lr_start; i++)				Leucine_Pattern += " ";

for (char i=1; i<mut_pos; i++)				Leucine_Pattern += "L------";
Leucine_Pattern += Sequence.at((Lr_start + (7 * ((int)mut_pos-1)))-1, 1);
for (char i = mut_pos; i<5; i++)			Leucine_Pattern += "------L";
for (int i=0; i<=(length-(Lr_start+(7*4+1))); i++)	Leucine_Pattern += " ";


String LZ_Pattern = "";
for (int i=1; i<LZ_start; i++)				LZ_Pattern += " ";
for (int i=1; i<=LZ_length;i++)			
	LZ_Pattern += "O";
for (int i=0; i<(length-(LZ_start+LZ_length-1)); i++)	LZ_Pattern += " ";

while (counter < length)
	{
       	int rest;
	int row_length = CharPerRow;

	rest = length - counter;
	if (rest < CharPerRow)		row_length = rest;
		
// hier kommen die Positionen

	int position_merker = position;

	while ((position-position_merker) < row_length)
		{
		int i;
		position++;
		printf("%d", position);
		position += stellen(position);
		for (i = stellen(position); i<9 && ((position-position_merker) < row_length); i++)	
			{
			putchar('-');
			position++;
			}
		if ((position-position_merker) <= row_length || i == 9)	putchar('-');
		}
	putchar('\n');

	position = position_merker;

// und dazu die Sequenz

	cout << Sequence.at(counter, row_length) << "\n";

// so, und jetzt das CoiledCoil

	for (int i=0; i<row_length; i++)
		{
		position++;
		if (position >= CC_start && position <= (CC_start+CC_length-1))	putchar('C');
		else								putchar(' ');
		}
	putchar('\n');

// ... Leucine repeats mit einer Mutation

	cout << Leucine_Pattern.at(counter, row_length) << "\n";

// ... Leucine Zipper annotation

	cout << LZ_Pattern.at(counter, row_length) << "\n\n";

	counter += CharPerRow;
	}
}
